@@ -1,3 +1,32 @@
+#' R Markdown Metadata
+#'
+#' Rmd files include a metadata section (typically located at the top of the
+#' file) that can specify (among other things) the title, author, and date of
+#' the document. Metadata adheres to the \href{https://yaml.org}{YAML} format
+#' and is delimited by lines containing three dashes (\code{---}). Here is an
+#' example metadata section:
+#' \preformatted{---
+#' title: "Crop Analysis Q3 2013"
+#' author: Martha Smith
+#' date: October 23rd, 2013
+#' ---
+#' }
+#' Note that the \code{title} field is quoted. This is because titles often
+#' contained embedded colons (\code{:}) and colons followed by a space need to
+#' be quoted in YAML.
+#' @details When title, author, and date metadata is provided it's used to
+#'   automatically create a title section within output documents. If you don't
+#'   want this section included in your document then you should remove the
+#'   corresponding metadata fields.
+#'
+#'   When generating PDF and Beamer output there are also a number of other
+#'   metadata fields that can be included to customize the appearance and theme
+#'   of PDF output. For more details see the documentation for
+#'   \code{\link{pdf_document}} and \code{\link{beamer_presentation}}.
+#' @name rmd_metadata
+NULL
+
+
 #' The YAML metadata of the current R Markdown document
 #'
 #' The object \code{metadata} stores the YAML metadata of the current R Markdown
@@ -10,6 +39,62 @@
 #' @export
 metadata <- list()
 
+
+#' Compiling R scripts to a notebook
+#'
+#' R Markdown can also compile R scripts to a notebook which includes
+#' commentary, source code, and script output. Notebooks can be compiled to any
+#' output format including HTML, PDF, and MS Word.
+#'
+#' @section Overview:
+#'   To compile a notebook from an R script you simply pass the script to
+#'   \code{\link{render}}. For example:
+#'   \preformatted{
+#' rmarkdown::render("analysis.R")
+#' rmarkdown::render("analysis.R", "pdf_document")
+#' }
+#'   The first call to \code{\link{render}} creates an HTML document, whereas
+#'   the second creates a PDF document.
+#'
+#'   By default the name of the script, username, and current date and time are
+#'   included in the header of the generated notebook. You can override this
+#'   default behavior by including explicit metadata in a specially formatted R
+#'   comment:
+#'   \preformatted{
+#' #' ---
+#' #' title: "Crop Analysis Q3 2013"
+#' #' author: "John Smith"
+#' #' date: "May 3rd, 2014"
+#' #' ---
+#' }
+#' @section Including Markdown:
+#'   Note that the R comment used above to add a title, author, and date
+#'   includes a single-quote as a special prefix character. This is a
+#'   \pkg{roxygen2} style comment, and it's actually possible to include many
+#'   such comments in an R script, all of which will be converted to markdown
+#'   content within the generated notebook. For example:
+#'   \preformatted{#' A script comment that includes **markdown** formatting.}
+#'   Rather than displaying as an R comment in the compiled notebook any
+#'   \pkg{roxygen2} style comment will be treated as markdown and rendered
+#'   accordingly.
+#' @section knitr Spin:
+#'   Including markdown within R comments is possible because \code{\link{render}}
+#'   calls the \code{\link[knitr:spin]{knitr spin}} function to convert the R
+#'   script to an Rmd file. The \code{spin} function also enables you to add
+#'    knitr
+#'   chunk options with another special comment prefix (\code{#+}).
+#'
+#'   Here's an example of a script that uses the various features of \code{spin}:
+#'
+#'   \url{https://github.com/yihui/knitr/blob/master/inst/examples/knitr-spin.R}
+#'
+#'   For more details on \code{knitr::spin} see the following documentation:
+#'
+#'   \url{http://yihui.org/knitr/demo/stitch/}
+#' @name compile_notebook
+NULL
+
+
 #' Render R Markdown
 #'
 #' Render the input file to the specified output format using pandoc. If the
@@ -67,6 +152,7 @@ metadata <- list()
 #' @seealso
 #' \link[knitr:knit]{knit}, \link{output_format},
 #' \href{http://johnmacfarlane.net/pandoc}{pandoc}
+#' @inheritParams default_output_format
 #' @param input The input file to be rendered. This can be an R script (.R),
 #' an R Markdown document (.Rmd), or a plain markdown document.
 #' @param output_format The R Markdown output format to convert to. The option
@@ -81,7 +167,11 @@ metadata <- list()
 #' output filename will be based on filename for the input file. If a filename
 #' is provided, a path to the output file can also be provided. Note that the
 #' \code{output_dir} option allows for specifying the output file path as well,
-#' however, if also specifying the path, the directory must exist.
+#' however, if also specifying the path, the directory must exist. If
+#' \code{output_file} is specified but does not have a file extension, an
+#' extension will be automatically added according to the output format. To
+#' avoid the automatic file extension, put the \code{output_file} value in
+#' \code{\link{I}()}, e.g., \code{I('my-output')}.
 #' @param output_dir The output directory for the rendered \code{output_file}.
 #' This allows for a choice of an alternate directory to which the output file
 #' should be written (the default output directory of that of the input file).
@@ -119,8 +209,7 @@ metadata <- list()
 #' @param run_pandoc An option for whether to run pandoc to convert Markdown
 #' output.
 #' @param quiet An option to suppress printing of the pandoc command line.
-#' @param encoding The encoding of the input file. See \code{\link{file}} for
-#' more information.
+#' @param encoding Ignored. The encoding is always assumed to be UTF-8.
 #' @return
 #'   When \code{run_pandoc = TRUE}, the compiled document is written into
 #'   the output file, and the path of the output file is returned. When
@@ -150,6 +239,7 @@ render <- function(input,
                    output_file = NULL,
                    output_dir = NULL,
                    output_options = NULL,
+                   output_yaml = NULL,
                    intermediates_dir = NULL,
                    knit_root_dir = NULL,
                    runtime =  c("auto", "static", "shiny", "shiny_prerendered"),
@@ -159,7 +249,7 @@ render <- function(input,
                    envir = parent.frame(),
                    run_pandoc = TRUE,
                    quiet = FALSE,
-                   encoding = getOption("encoding")) {
+                   encoding = "UTF-8") {
 
   perf_timer_start("render")
 
@@ -177,7 +267,7 @@ render <- function(input,
 
   # check for "all" output formats
   if (identical(output_format, "all")) {
-    output_format <- enumerate_output_formats(input, envir, encoding)
+    output_format <- enumerate_output_formats(input)
     if (is.null(output_format))
       output_format <- "html_document"
   }
@@ -186,12 +276,12 @@ render <- function(input,
   # then recursively call this function with each format by name
   if (is.character(output_format) && length(output_format) > 1) {
     outputs <- character()
-    for (format in output_format) {
+    for (i in seq_along(output_format)) {
       # the output_file argument is intentionally ignored (we can't give
       # the same name to each rendered output); copy the rest by name
       output <- render(input = input,
-                       output_format = format,
-                       output_file = NULL,
+                       output_format = output_format[i],
+                       output_file = output_file[i],
                        output_dir = output_dir,
                        output_options = output_options,
                        intermediates_dir = intermediates_dir,
@@ -202,14 +292,9 @@ render <- function(input,
                        knit_meta = knit_meta,
                        envir = envir,
                        run_pandoc = run_pandoc,
-                       quiet = quiet,
-                       encoding = encoding)
+                       quiet = quiet)
       outputs <- c(outputs, output)
     }
-    if (length(output_file) > 1) {
-      file.rename(outputs, output_file)
-      outputs <- output_file
-    }
     return(invisible(outputs))
   }
 
@@ -284,7 +369,7 @@ render <- function(input,
   force(knit_root_dir)
 
   # execute within the input file's directory
-  oldwd <- setwd(dirname(tools::file_path_as_absolute(input)))
+  oldwd <- setwd(dirname(abs_path(input)))
   on.exit(setwd(oldwd), add = TRUE)
 
   # reset the name of the input file to be relative and calculate variations
@@ -319,29 +404,28 @@ render <- function(input,
       '---\n',
       'title: "', input, '"\n',
       'author: "', Sys.info()[["user"]], '"\n',
-      'date: "', date(), '"\n',
+      'date: "', Sys.Date(), '"\n',
       '---\n'
     , sep = "")
-    if (!identical(encoding, "native.enc"))
-      metadata <- iconv(metadata, to = encoding)
-    cat(metadata, file = knit_input, append = TRUE)
+    input_lines <- read_utf8(knit_input)
+    write_utf8(c(input_lines, metadata), knit_input)
   }
 
   # read the input file
-  input_lines <- read_utf8(knit_input, encoding)
+  input_lines <- read_utf8(knit_input)
 
   # read the yaml front matter
-  yaml_front_matter <- parse_yaml_front_matter(input_lines)
+  front_matter <- parse_yaml_front_matter(input_lines)
 
   # metadata to be attached to the returned value of render() as an attribute
   old_output_metadata <- output_metadata$get()
   on.exit(output_metadata$restore(old_output_metadata), add = TRUE)
-  output_metadata$restore(as.list(yaml_front_matter[['rmd_output_metadata']]))
+  output_metadata$restore(as.list(front_matter[['rmd_output_metadata']]))
 
   # if this is shiny_prerendered then modify the output format to
   # be single-page and to output dependencies to the shiny.dep file
   shiny_prerendered_dependencies <- list()
-  if (requires_knit && is_shiny_prerendered(yaml_front_matter$runtime)) {
+  if (requires_knit && is_shiny_prerendered(front_matter$runtime)) {
 
     # first validate that the user hasn't passed an already created output_format
     if (is_output_format(output_format)) {
@@ -374,15 +458,18 @@ render <- function(input,
     output_format <- output_format_from_yaml_front_matter(input_lines,
                                                           output_options,
                                                           output_format,
-                                                          encoding = encoding)
+                                                          output_yaml)
     output_format <- create_output_format(output_format$name,
                                           output_format$options)
   }
   pandoc_to <- output_format$pandoc$to
 
   # generate outpout file based on input filename
-  if (is.null(output_file))
-    output_file <- pandoc_output_file(input, output_format$pandoc)
+  output_auto <- pandoc_output_file(input, output_format$pandoc)
+  if (is.null(output_file) || is.na(output_file)) output_file <- output_auto else {
+    if (!inherits(output_file, "AsIs") && xfun::file_ext(output_file) == "")
+      output_file <- paste(output_file, xfun::file_ext(output_auto), sep = ".")
+  }
 
   # if an output_dir was specified then concatenate it with the output file
   if (!is.null(output_dir)) {
@@ -392,7 +479,7 @@ render <- function(input,
 
   # Stop the render process early if the output directory does not exist
   if (!dir_exists(output_dir)) {
-    stop("The directory '", output_dir, "') does not not exist.",
+    stop("The directory '", output_dir, "' does not not exist.",
          call. = FALSE)
   }
 
@@ -409,7 +496,6 @@ render <- function(input,
       !is.null(output_format$intermediates_generator)) {
     intermediates <- c(intermediates,
                        output_format$intermediates_generator(original_input,
-                                                             encoding,
                                                              intermediates_dir))
   }
 
@@ -425,12 +511,7 @@ render <- function(input,
   # presume that we're rendering as a static document unless specified
   # otherwise in the parameters
   runtime <- match.arg(runtime)
-  if (identical(runtime, "auto")) {
-    if (!is.null(yaml_front_matter$runtime))
-      runtime <- yaml_front_matter$runtime
-    else
-      runtime <- "static"
-  }
+  if (identical(runtime, "auto")) runtime <- front_matter$runtime %||% "static"
 
   # set df_print
   context <- render_context()
@@ -444,10 +525,10 @@ render <- function(input,
   # function used to call post_knit handler
   call_post_knit_handler <- function() {
     if (!is.null(output_format$post_knit)) {
-      post_knit_extra_args <- output_format$post_knit(yaml_front_matter,
+      post_knit_extra_args <- output_format$post_knit(front_matter,
                                                       knit_input,
                                                       runtime,
-                                                      encoding = encoding)
+                                                      encoding = 'UTF-8')
     } else {
       post_knit_extra_args <- NULL
     }
@@ -502,7 +583,7 @@ render <- function(input,
     # read root directory from argument (has precedence) or front matter
     root_dir <- knit_root_dir
     if (is.null(root_dir))
-      root_dir <- yaml_front_matter$knit_root_dir
+      root_dir <- front_matter$knit_root_dir
     if (!is.null(root_dir))
       knitr::opts_knit$set(root.dir = root_dir)
 
@@ -539,7 +620,7 @@ render <- function(input,
       shiny_prerendered_remove_uncached_data(original_input)
 
       # set the cache option hook and evaluate hook
-      knitr::opts_hooks$set(label = shiny_prerendered_option_hook(original_input,encoding))
+      knitr::opts_hooks$set(label = shiny_prerendered_option_hook(original_input))
       knitr::knit_hooks$set(evaluate = shiny_prerendered_evaluate_hook(original_input))
     }
 
@@ -582,7 +663,7 @@ render <- function(input,
     # make the params available within the knit environment
     # (only do this if there are parameters in the front matter
     # so we don't require recent knitr for all users)
-    if (!is.null(yaml_front_matter$params)) {
+    if (!is.null(front_matter$params)) {
 
       params <- knit_params_get(input_lines, params)
 
@@ -618,9 +699,8 @@ render <- function(input,
       }, add = TRUE)
     }
 
-    # make the yaml_front_matter available as 'metadata' within the
-    # knit environment (unless it is already defined there in which case
-    # we emit a warning)
+    # make the front_matter available as 'metadata' within the knit environment
+    # (unless it is already defined there, in which case we emit a warning)
     env <- environment(render)
     metadata_this <- env$metadata
     do.call("unlockBinding", list("metadata", env))
@@ -631,7 +711,7 @@ render <- function(input,
       env$metadata <- metadata_this
       lockBinding("metadata", env)
     }, add = TRUE)
-    env$metadata <- yaml_front_matter
+    env$metadata <- front_matter
 
     # call onKnit hooks (normalize to list)
     sapply(as.list(getHook("rmarkdown.onKnit")), function(hook) {
@@ -649,11 +729,12 @@ render <- function(input,
     input <- knitr::knit(knit_input,
                          knit_output,
                          envir = envir,
-                         quiet = quiet,
-                         encoding = encoding)
+                         quiet = quiet)
 
     perf_timer_stop("knitr")
 
+    front_matter <- yaml_front_matter(input)
+
     # call post_knit handler
     output_format$pandoc$args <- call_post_knit_handler()
 
@@ -664,7 +745,7 @@ render <- function(input,
     }
 
     # pull out shiny_prerendered_contexts and append them as script tags
-    shiny_prerendered_append_contexts(runtime, input, encoding)
+    shiny_prerendered_append_contexts(runtime, input)
 
     # collect remaining knit_meta
     knit_meta <- knit_meta_reset()
@@ -677,13 +758,13 @@ render <- function(input,
   if (!(is_pandoc_to_html(output_format$pandoc) ||
         identical(tolower(tools::file_ext(output_file)), "html")))  {
     if (has_html_dependencies(knit_meta)) {
-      if (!isTRUE(yaml_front_matter$always_allow_html)) {
+      if (!isTRUE(front_matter$always_allow_html)) {
         stop("Functions that produce HTML output found in document targeting ",
              pandoc_to, " output.\nPlease change the output type ",
              "of this document to HTML. Alternatively, you can allow\n",
              "HTML output in non-HTML formats by adding this option to the YAML front",
              "-matter of\nyour rmarkdown file:\n\n",
-             "  always_allow_html: yes\n\n",
+             "  always_allow_html: true\n\n",
              "Note however that the HTML output will not be visible in non-HTML formats.\n\n",
              call. = FALSE)
       }
@@ -696,13 +777,28 @@ render <- function(input,
   }
 
   # clean the files_dir if we've either been asking to clean supporting files or
-  # the knitr cache is active
-  if (output_format$clean_supporting && (is.null(cache_dir) || !dir_exists(cache_dir)))
-      intermediates <- c(intermediates, files_dir)
+  # the knitr cache is active; clean the figure-* dir instead of the whole
+  # files_dir if other subdirs are generated by another format and still needed:
+  # https://github.com/rstudio/rmarkdown/issues/1472 and also #1503
+  if (output_format$clean_supporting && !dir_exists(cache_dir)) {
+    # unlink does not support / at the end of file path
+    fig_path <- gsub("/$", "", knitr::opts_chunk$get('fig.path'))
 
-  # read the input text as UTF-8 then write it back out
-  input_text <- read_utf8(input, encoding)
-  write_utf8(input_text, utf8_input)
+    # existing figure folder(s), can be character(0)
+    # if no figure is generated, clean the whole files_dir (#1664)
+    files_dir_fig <- list.files(files_dir, '^figure-.+')
+
+    intermediates <- c(
+      intermediates,
+      if (length(files_dir_fig) < 1 || identical(files_dir_fig, basename(fig_path))) {
+        files_dir
+      } else {
+        fig_path
+      }
+    )
+  }
+
+  file.copy(input, utf8_input, overwrite = TRUE)
 
   if (run_pandoc) {
 
@@ -710,7 +806,7 @@ render <- function(input,
 
     # call any pre_processor
     if (!is.null(output_format$pre_processor)) {
-      extra_args <- output_format$pre_processor(yaml_front_matter,
+      extra_args <- output_format$pre_processor(front_matter,
                                                 utf8_input,
                                                 runtime,
                                                 knit_meta,
@@ -742,7 +838,7 @@ render <- function(input,
         figures_dir <- gsub('/$', '', knitr::opts_chunk$get("fig.path"))
         files <- list.files(figures_dir, full.names = TRUE, recursive = TRUE)
         # https://github.com/rstudio/rmarkdown/issues/1358
-        if (citeproc) files <- c(files, yaml_front_matter[['bibliography']])
+        if (citeproc) files <- c(files, front_matter[['bibliography']])
         for (f in files) {
           intermediates <<- c(intermediates, copy_file_with_dir(f, intermediates_dir))
         }
@@ -807,13 +903,16 @@ render <- function(input,
     texfile <- file_with_ext(output_file, "tex")
     # determine whether we need to run citeproc (based on whether we have
     # references in the input)
-    run_citeproc <- citeproc_required(yaml_front_matter, input_lines)
+    run_citeproc <- citeproc_required(front_matter, input_lines)
     # if the output format is LaTeX, first convert .md to .tex, and then convert
     # .tex to .pdf via latexmk() if PDF output is requested (in rmarkdown <=
     # v1.8, we used to call Pandoc to convert .md to .tex and .pdf separately)
     if (output_format$pandoc$keep_tex || knitr::is_latex_output()) {
       # do not use pandoc-citeproc if needs to build bibliography
       convert(texfile, run_citeproc && !need_bibtex)
+      # patch the .tex output generated from the default Pandoc LaTeX template
+      if (!("--template" %in% output_format$pandoc$args)) patch_tex_output(texfile)
+      fix_horiz_rule(texfile)
       # unless the output file has the extension .tex, we assume it is PDF
       if (!grepl('[.]tex$', output_file)) {
         latexmk(texfile, output_format$pandoc$latex_engine, '--biblatex' %in% output_format$pandoc$args)
@@ -840,7 +939,7 @@ render <- function(input,
 
     # if there is a post-processor then call it
     if (!is.null(output_format$post_processor))
-      output_file <- output_format$post_processor(yaml_front_matter,
+      output_file <- output_format$post_processor(front_matter,
                                                   utf8_input,
                                                   output_file,
                                                   clean,
@@ -863,7 +962,7 @@ render <- function(input,
 
   if (run_pandoc) {
     # return the full path to the output file
-    output_file <- tools::file_path_as_absolute(output_file)
+    output_file <- abs_path(output_file)
     # attach the metadata specified as rmd_output_metadata in YAML
     if (length(output_meta <- output_metadata$get()))
       attr(output_file, 'rmd_output_metadata') <- output_meta
